package crystal.cfg;

import java.io.*;
import java.util.*;

/** Information about the CFG for a function */
public class CFGInfo implements Serializable {
	private static final long serialVersionUID = -7950689232967234029L;

	CFGInfo(CFGNode entryNode, CFGNode exitNode) {
		this.entryNode = entryNode;
		this.exitNode  = exitNode;
	}

	public CFGNode entryNode;
	public CFGNode exitNode;

	static private Stack  <CFGNode> work = new Stack<CFGNode>();
	static private HashSet<CFGNode> marks = new HashSet<CFGNode>();

	public List<CFGNode> getAllNodes() {
		List <CFGNode> list = new ArrayList<CFGNode>();
		marks.clear();
		work.clear();
		work.push(entryNode);
		while (!work.empty()) {
			CFGNode node = work.pop();
			if (!marks.contains(node)) {
				marks.add(node);
				list.add(node);
				for (int i = 0; i < node.numSuccs(); i++) {
					CFGNode next = node.getSucc(i);
					if (next != null)
						work.push(next);
				}
			}
		}
		return list;
	}
}
