package crystal.emitc;

import java.io.*;

import crystal.general.*;

public class EmitC {
	
	public static void run(boolean verbose, boolean comments) {
		HashSymtab globals = Symtab.getGlobalScope();
		System.setProperty("line.separator", "\n");

		if (verbose)
			System.out.println();
		
		for (Symtab filetab: globals.getKids()) {
			String infile = ((FileSymtab)filetab).getFileName();
			String outfile = adjustSuffix(infile);
				
			PrintStream ps = null;
			try {
				ps = new PrintStream(outfile);
			} catch (FileNotFoundException e) {
		    	System.err.println("File not found: \"" + outfile + "\"");
		    	System.exit(1);
			}

			if (verbose)
				System.out.println("\twriting file \"" + outfile + "\" ...");
	
			if (comments)
				ps.println("/* This file has been automatically generated " +
						"by Crystal. */\n" +
						"/* Original source file: \"" + infile + "\" */\n");
			
			SymtabPrinter.printGlobal(filetab, ps, "");

			if (comments)
				ps.println("\n/* File scope starts here. */\n");
			
			SymtabPrinter.printSymtab((HashSymtab)filetab, ps, "");
			ps.println();

			for (Function fsym : Symtab.getFunctions(infile)) 
				fsym.printFunction(ps,  fsym.getCFG() != null);
			ps.println();				
		}
		
		if (verbose)
			System.out.print("Written all output C files. ");	
	}
	
	public static String adjustSuffix(String file) {
		int i = file.lastIndexOf('.');
		if (i > 0) file = file.substring(0,i);
		return file + ".out.c";
	}
}

