package crystal.projects.leakcontradictor;

import crystal.core.*;
import crystal.analysis.ipa.*;
import crystal.analysis.pointer.*;

import java.util.*;


/** The secondary portion of the abstraction element. */
public class S implements Secondary<S> {

	HashSet<CoreExpr> hit;
	HashSet<CoreExpr> miss;
	HashMap<Region, CoreExpr> regions;
	boolean guaranteedExternRef;
	int depth;
	int calls;


	public S() {
		 hit      = new HashSet<CoreExpr>();
		 miss     = new HashSet<CoreExpr>();
		 regions  = new HashMap<Region, CoreExpr>();
		 guaranteedExternRef = false;
	}

	public S(HashSet<CoreExpr> h, HashSet<CoreExpr> m,
             HashMap<Region,  CoreExpr>  r) {
		hit  = h;
		miss = m;
		regions  = r;
	}

	public boolean merge(S other) {
		boolean changed = false;
		changed = hit.retainAll(other.hit)  || changed;
		changed = miss.retainAll(other.miss) || changed;

		/* Merge in region info.. */
		Iterator<Map.Entry<Region, CoreExpr>> i = 
			other.regions.entrySet().iterator();
		while (i.hasNext()) {
			Map.Entry<Region, CoreExpr> info = i.next();
			Region r = info.getKey();
			CoreExpr oneRef = info.getValue();
			CoreExpr ourRef = regions.get(r);

			if (oneRef != ourRef) {
				changed = true;

				/* We didn't have this region -> use others */
				if (ourRef == null) {
					regions.put(r, oneRef);
				} else {
					/* We had it, other did to, and they're different -> T*/
					regions.put(r, Analysis.nil);
				}
			}
		}

		changed = changed || (guaranteedExternRef && !other.guaranteedExternRef);
		guaranteedExternRef &= other.guaranteedExternRef;

		return changed;
	}

	public boolean equals(S other) {
		if (!hit.equals(other.hit))
			return false;
		if (!miss.equals(other.miss))
			return false;
		if (!regions.equals(other.regions))
			return false;
		return guaranteedExternRef == other.guaranteedExternRef;
	}

	@SuppressWarnings("unchecked")
	public S clone() {
		S copy = new S();
		copy.hit.addAll (hit);
		copy.miss.addAll(miss);
		copy.regions = (HashMap<Region, CoreExpr>)regions.clone();
		copy.guaranteedExternRef = guaranteedExternRef;
		copy.depth = depth;
		copy.calls = calls;
		return copy;
	}

	String toStringDot() {
		return regions + "|+" + hit + "|-" + miss + "|e?" + guaranteedExternRef;
	}
}
